/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.sim.sim.simulationsstrecke;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.sim.sim.EmpfaengerMitAnmeldung;

/**
 * Die Klasse kapselt die Startskript-Informationen und den Parameterdatensatz zur 
 * Spezifikation der Simulationsstrecke
 *
 * @version $Revision: 1.3 $ / $Date: 2008/10/31 13:30:32 $ / ($Author: Sans $)
 *
 * @author Dambach-Werke GmbH
 * @author Stefan Sans
 *
 */
public class SimulationsStrecke implements EmpfaengerMitAnmeldung
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  private String _name = "";
  
  private Map<String, String> _startSkriptInfo;
  
  private ClientDavInterface _connection;
  
  private SystemObject _objekt;
  
  private Array _parameterSatz;
  
  
  /**
   * Erzeugt eine neue Instanz vom Typ Simulationsstrecke
   * 
   * @param obj das dynamische Simulationsstrecken-Objekt
   * @param con die Datenverteiler-Verbindung
   */
  public SimulationsStrecke(SystemObject obj, ClientDavInterface con)
  {
    _objekt = obj;
    _connection = con;
    
    _startSkriptInfo = new LinkedHashMap<String, String>();
  }
  
  /**
   * Bestimmt den Namen der Simulatinsstrecke
   * 
   * @return den Namern der Simulationsstrecke
   */
  public String getName()
  {
    return _name;
  }
  
  @Override
  public String toString()
  {
    return _name;
  }
  
 /**
  * Bestimmt die Startskript-Informationen der Simulationsstrecke
  * 
  * @return eine Map: Schlssel ist jeweils die PID 
  * des Rechnerobjkets, Wert ist die Startskript-Spezifikation
  */
  public Map<String, String> getStartSkriptInfo()
  {
    return Collections.unmodifiableMap(_startSkriptInfo);
  }

  /**
   * Bestimmt den Parameter-Datensatz der Eingangsdaten der Simualtionsstrecke 
   * 
   * @return ein Array mit dem Parameter-Datensatz der Eingangsdaten 
   */
  public Array getParameterSatz()
  {
    return _parameterSatz;
  }
    
  public SystemObject getObjekt()
  {
    return _objekt;
  }
  
  public ClientDavInterface getConnection()
  {
    return _connection;
  }
  
  public void anmelden()
  {
    ReceiverRole empfaengerrolle = ReceiverRole.receiver();
    ReceiveOptions anmeldeoptionen = ReceiveOptions.normal();
    Aspect asp = getObjekt().getDataModel().getAspect("asp.parameterSoll");  
    AttributeGroup atg =  getObjekt().getDataModel().getAttributeGroup("atg.simulationsStreckenBeschreibung");
    short sv = 0;
    DataDescription datenbeschreibung = new DataDescription(atg, asp, sv);
    getConnection().subscribeReceiver(this, getObjekt(), datenbeschreibung, 
        anmeldeoptionen, empfaengerrolle);
  }
  
  public void update(ResultData[] resultData)
  {
    for (ResultData data : resultData)
    {
      _debug.finest("Daten empfangen:\n" + data);
      
      if (data.getDataState() == DataState.DATA)
      {
        Aspect asp = data.getDataDescription().getAspect();
        AttributeGroup atg = data.getDataDescription().getAttributeGroup();
        short sv = data.getDataDescription().getSimulationVariant();
        
        _debug.fine("Daten empfangen mit atg/asp/sv: " + atg.getPid() + "/" +
            asp.getPid() + "/" + sv + ":\n" + data);
        
        if (sv == 0 && asp.getPid().equals("asp.parameterSoll") &&
            atg.getPid().equals("atg.simulationsStreckenBeschreibung"))
        {
          _debug.fine("Bearbeite Daten mit atg/asp/sv: " + atg.getPid() + "/" +
              asp.getPid() + "/" + sv + ":\n" + data);
          
          _name = data.getData().getItem("Name").asTextValue().getText();
          _debug.fine("Parametrierung simulationsStreckenBeschreibung fr Simulationsstrecke " + getName());
          
          _startSkriptInfo.clear();
          Array array = data.getData().getArray("StartInfo");
          for (int i = 0; i < array.getLength(); i += 1)
          {
            Data datenSatz = array.getItem(i);
            SystemObject rechner = datenSatz.getItem("Rechner").asReferenceValue().getSystemObject();
            if (rechner != null)
            {
              String startSkript = datenSatz.getItem("Startskript").asTextValue().getText();
              _startSkriptInfo.put(rechner.getPid(), startSkript);
              _debug.fine("(" + rechner.getPid() + "," + startSkript + ") in Startliste fr Simulation "
                  + getName() + " aufgenommen");
            }
            else
            {
              _debug.error("Ungltiger Eintrag in " + atg + ", Array 'StartInfo', " +
                  i + 1 + ". Eintrag - Referenz " + datenSatz.getItem("Rechner").asReferenceValue());
            }
          }
          
          _parameterSatz = data.getData().getArray("ParameterSatz");
        }
      }
    }  
  }
  
}
