/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.sim.sim.verwaltung;

/**
 * Singleton zum Verwalten von Simulationsvarianten
 * 
 * @author Dambach Werke GmbH
 * @author Stefan Sans
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:30:32 $ / ($Author: Sans $)
 */
public class SimulationsVarianten 
{
  /**
   * Feld fr die Zustnde "frei"/"belegt" jeder mglichen Simulationsvarinate
   */
	private boolean[] _werteArray = null;
	
  /**
   * Privater Default-Konstruktor
   */
	private SimulationsVarianten()
	{
	  _werteArray = new boolean[0];	
	}
	
  /**
   * Hier wird die einzige Instanz dieser Klasse gespeichert
   */
	private static SimulationsVarianten _simulationsvarianten = null;
	
	/**
	 * Bestimmt die einzige Instanz fr diese Klasse 
	 * @return die einzige Instanz fr diese Klasse
	 */
	public static SimulationsVarianten getInstanz()
	{
		if (_simulationsvarianten == null)
		{
			_simulationsvarianten = new SimulationsVarianten();
		}
		return _simulationsvarianten;
	}
	
  /**
   * Setzt die maximal zulssige Simulationsvariante. Diese Methode MUSS
   * vor Aufruf andererer Methoden mindestens einmal mit einem Wert > 0 
   * aufgerufen werden, sonst ist die maximale Simulationsvariante 0.
   * 
   * @param maximum die maximale Simulationsvariante
   * @throws IllegalArgumentException, wenn der bergebene Wert < 0 ist
   */
  public void setMaximum(short maximum)
  {
    if (maximum < 0)
    {
      throw new IllegalArgumentException("Die maximale Simulationsvariante muss >= 0 sein");
    }
    
    boolean[] werteArray = new boolean[maximum];
    for (int i = 0; i < _werteArray.length && i < werteArray.length; i += 1)
    {
      werteArray[i] = werteArray[i];
    }
    
    _werteArray = werteArray;
  }
  
	/**
	 * Setzt eine Simulationsvariante auf 'frei'
	 * @param simulationsvariante die Simulationsvariante zwischen 1 und 999, 
	 * die auf 'frei' gesetzt werden soll
	 */
	public void setFrei(short simulationsvariante)
	{
		if (simulationsvariante > 0 && simulationsvariante < _werteArray.length + 1)
		{
			_werteArray[simulationsvariante - 1] = false;
		}
		// else
		// tue nichts
	}

	/**
	 * Setzt eine Simulationsvariante auf 'belegt'
	 * @param simulationsvariante die Simulationsvariante zwischen 1 und 999, 
	 * die auf 'belegt' gesetzt werden soll
	 */
	public void setBelegt(short simulationsvariante)
	{
		if (simulationsvariante > 0 && simulationsvariante < _werteArray.length + 1)
		{
			_werteArray[simulationsvariante - 1] = true;
		}
		// else
		// tue nichts
	}

	/**
	 * Prft, ob eine Simulationsvariante frei ist
	 * @param simulationsvariante die Simulationsvariante zwischen 1 und 999, 
	 * die geprft werden soll
	 * @return true wenn die Simulationsvariante frei ist, false sonst. Wenn 
	 * eine Simulationsvariante ausserhalb von 1..999 bergeben wird, wird 
	 * false zurckgegegen
	 */
	public boolean isFrei(short simulationsvariante)
	{
		if (simulationsvariante > 0 && simulationsvariante < _werteArray.length + 1)
		{
			return (_werteArray[simulationsvariante - 1] == false);
		}
		return false;
	}

	/**
	 * Prft, ob eine Simulationsvariante belegt ist
	 * @param simulationsvariante die Simulationsvariante zwischen 1 und 999, 
	 * die geprft werden soll
	 * @return true wenn die Simulationsvariante belegt ist, false sonst. Wenn 
	 * eine Simulationsvariante ausserhalb von 1..999 bergeben wird, wird true 
	 * zurckgegegen
	 */
	public boolean isBelegt(short simulationsvariante)
	{
		if (simulationsvariante > 0 && simulationsvariante < _werteArray.length + 1)
		{
			return (_werteArray[simulationsvariante - 1] == true);
		}
		return true;
	}
	
	/**
	 * Liefer die nchst-niedrigste freie Simulataionsvariante
	 * @return die nchst-niedrigste freie Simulataionsvariante freie 
	 * Simualtionsvariante oder 0, falls keine Simulationsvariante frei ist.
	 */
	public short getNiedrigsteFreie()
	{
		for (short i = 0; i < _werteArray.length; i += 1)
		{
			if (_werteArray[i] == false)
			{
				return (short)(i + 1);
			}
		}
		return 0;
	}
}
